package rcliper_plot;

#! /use/bin/perl -w

use 5.008;
use strict;
use IO::File;

require "rcliper_config.pl";
require $Config::generalCode;

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Program Name: rcliper_plot.pl
#
# Language: Perl 5.8.x
#
# Version: 3.1
#
# Purpose: Create a PS file and a GEMPAK grid from a
#		   RCLIPER output text file.
#
# Called by: rcliper.pl
#
#---------------------------------------------------
#  Data Structure:
#
#	"psFile" 			=> "",
#	"gridFile" 			=> "",
#	"nmapFile"			=> "",
#	"nmap_boundaries" 		=> "0.000;-190.000;70.000;-20.000",
#	"nmap_extent" 			=> "341;141",
#	"device"			=> "psc",
#	"proj"      			=> "CED/0;0;0",
#	"totalgrids"  			=> 100,
#	"ivcord"			=> "none",
#	"packing"			=>	" ",
#	"title"				=>	"	",
#	"shell"				=>  "/usr/bin/sh",
#	"cronProfile"			=>	$cronprofile
#
#	psFile 	 			Name of the Postscript file 
#	gridFile			Name of the GEMPAK grid file
#	nmapFile			Name of the nmap GEMPAK grid
#	nmap_boundaries    		The geogpraphic extent of NMAP grid
#	nmap_extent          		The resolution of the GEMPAK grid for display in NMAP
#	device				Set to create postscript file in gempak
#	proj				Projection for the postscript file
#	totalgrids	 		Total number of potential grids that can be put in a GEMPAK grid
#	ivcord				GEMPAK variable
#	packing				GEMPAK variable
#	title				Title on the bottom of the Postscript plot
#	cronProfile 			Name of a .profile to set GEMPAK environment variables 
#	shell				Name and path to local UNIX shell
#	asctogem			Absolute path of the asctogem executable
#---------------------------------------------------
# Subroutines:
#
### Subroutine: plotRcliper
#
# Purpose:  Manages the process to create a grid or a ps file
#			of the RCLIPER data.
#
### Subroutine: createPlot	
#
# Purpose:  Uses a GEMPAK to convert a text file into a GEMPAK 
#			grid.
#
# Note:		ASCTOGEM adds additional functionality to gempak
#           which allows a user to convert a regularly spaced ascii file
#           of a specified format into a gempak grid.
#
#           The asctogem routine is not part of the gempak distribution.
#           The code needs to be placed in a known directory.
#
### Subroutine: addPlot
#
# Purpose:  Adds a new GEMPAK grid to an existing GEMPAK grid.
#
### Subroutine: asctogem		
#
# Purpose:  This includes additional functionality to gempak
#           which allows a user to convert a regularly spaced ascii file
#           of a specified format into a gempak grid.
#
#           The asctogem routine is not part of the gempak distribution.
#           The code needs to be placed in a known directory.
#
### Subroutine: gdplot2	
#
# Purpose: Plots the data in the gempak grid by contouring
#		   it and filling in the contours with specified colors.
#
### Subroutine: gpmap		
#
# Purpose: Adds the title to the plot.
#
### Subroutine: atest	
#
# Purpose: Add the track points to the plot in red with x's at the points
#
#------------------------------------------------------
# Access Subroutines
#
### Subroutine: getNmapName
# 
# 	Purpose: Return the name of the NMAP grid file
#			to the caller.
#
### Subroutine: getGridName
# 
# 	Purpose: Return the name of the grid file
#			to the caller.
#
### Subroutine: getPSname
# 
# 	Purpose: Return the name of the Postscript file
#			to the caller.
#
#---------------------------------------------------
# Information:
#    Joint Hurricane Testbed/USWRP project
#    National Hurricane Center / Tropical Prediction Center
#    Department of Commerce - NOAA / NWS
#
# Category: JHT - RCLIPER 
#
# M. Mainelli  09/01 Initial Creation       
# M. Mainelli  06/02 updated  
# A. Krautkramer 09/02 converted ksh script into perl    
# A. Krautkramer 4/03 automation of PS creation     
#			and the creation of gempak grid for viewing
#			in nmap2 
# A. Krautkramer 10/03 restructing code   
# A. Krautkramer 10/04 
#
#		Modified program so that each forecast hour grid
#		can be added to one gempak grid file.
#
#		line 189 -> changed the name assigned to a nmap file
#		line 165 -> changed the max number of grids that 
#				can be include in any RCLIPER grid
#
# A. Krautkramer 11/17/2004
#
#		Modified code so that each forecast hour grid will be added
#		under a different parameter name.
#
#		line 176 -> added the 'parameter' varaible to the plot data
#			    structure.  This variable will hold the name of the 
#			   gempak grid parameter used in the data file.
#
#		line 182 -> Set the 'parameter' hash value to the value of the
#			gempak parameter value assigned in the rcliper_data.pl program.
#				
#		line 841 -> Used the dynamically set parameter hash value to add
#			data grids to a gempak grid instead of using the static 'RAIN'
#			parameter name.
#
#		line 630 -> Used the dynamically set parameter hash value to plot
#			 the correct gempak data
#
# A. Krautkramer 11/19/2004
#
# 		Modified code so that a dashed line will appear to demarkate the
#		0.5 inch  contour.  This included adding a second call to GDPLOT2
#
#		Changed the name given to the gridFile so that the name would take
#		into account the time period over which the rain is accumulated.
#
# 		Added a new entry to the $rPlot has called parameter.  This
#		variable will be the name used by GEMPAK to identify grids.
#		Currently, the parameter value can be either RN24, RN48 or RN72.
#
#		Commented out the command that deletes the nmap grid file in the working
#		directory before creating a new grid in the plotRcliper function.  
#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


#--------------------------------------------------------
# Subroutine: new
#
# Purpose:  Initialized the RCLIPER Plot object
#
# Input: $_[0] - RCLIPER Data object
#	 $_[1] - Forecast hour  
#
# Output: RCLIPER Plot Object
#-----------------------------------------------------------
sub new {


	#=================================
	# *** Define Variable ***
	#=================================

	
	my ($rRcliper, $fHour) = @_;


	my $rPlot = {
	
				"psFile" 			=> "",
				"gridFile" 			=> "",
				"nmapFile"			=> "",
				"nmap_boundaries" 		=> "0.000;-190.000;70.000;-20.000",
				"nmap_extent" 			=> "341;141",
				"device"			=> "psc",
				"proj"      			=> "CED/0;0;0",
				"totalgrids"  			=> 300,
				"ivcord"			=> "none",
				"packing"			=>	" ",
				"title"				=>	"	",
				"shell"				=>  "/bin/sh",
				"cronProfile"			=>	$Config::cronprofile,
				"asctogem"			=>	$Config::asctogem,
				"parameter"			=> ""
				
			};
			
	bless $rPlot, "rcliper_plot";	
	
	$$rPlot{parameter} = $rRcliper->getParameter();
		
	#=================================		
	#*** Initialize file names
	#=================================	
		
	if ($fHour =~ /[0-9]/) {
		
		$rPlot->{psFile} = substr($rRcliper->getStormID(), 0, 4) . "_" . $rRcliper->getTime() . "f" . $fHour . "_rcliper.ps";	
		#$rPlot->{gridFile} = "temp_" . $rRcliper->getTime() $fHour . "_" . $rRcliper->getStormID();
		$rPlot->{gridFile} = "rcliper_" . $rRcliper->getTime() . "f" . $fHour . "_" . $rRcliper->getStormID();
		
		#$rPlot->{nmapFile} = "rcliper_" . $rRcliper->getDate() . "00f$fHour\_" . $rRcliper->getStormID();
		#$rPlot->{nmapFile} = "rcliper_" . $rRcliper->getDate() . "00\_" . $rRcliper->getStormID() . "f$fHour";
		$rPlot->{nmapFile} = "rcliper_" . $rRcliper->getDate() . "00\_" . $rRcliper->getStormID();
			
		$rPlot->{title} =  "$fHour hr forecast RCLIPER " . $rRcliper->getTime() . " SFC RAIN (INCHES) for "  . $rRcliper->getStormID();
			
	}
	else {
		
		$rPlot->{psFile} = $rRcliper->getStormID() . "_rcliper.ps";	
		$rPlot->{gridFile} = "temp_" . $rRcliper->getStormID();
		$rPlot->{nmapFile} = "rcliper_" . $rRcliper->getDate() . "00_" . $rRcliper->getStormID();	
		$rPlot->{title} =  "RCLIPER SFC RAIN (INCHES) for " .  $rRcliper->getName();
	
	}
	

		
	return $rPlot;

}

#--------------------------------------------------------
# Subroutine: plotRcliper
#
# Purpose:  Manages the process to create a grid or a ps file
#			of the RCLIPER data.
#
# Input: $_[0] - Plot object
#		 $_[1] - Rcliper object
#		 $_[2] - the type of plot the user wants to create
#				 valid options "ps" or "grid"
#				 	"ps" - create a postscript file of the RCLIPER data
#					"grid" - create a grid that can be viewed in NMAP
#		 $_[3] - file handle to log
#
# Output:	1 - success
#			-1 - Error: Grid file not found 
#			-2 - Error: Incorrect type used
#-----------------------------------------------------------
sub plotRcliper {


	#=================================
	# *** Define Variable ***
	#=================================

	
	my ($rPlot, $rRcliper, $type, $LOG) = @_;
	my $returnCode;
	
	
	#================================================
	# *** Check if the rcliper text output exists
	#================================================
	
	
	$returnCode = generalUtils::isFile($Config::workDir, $rRcliper->getData());

	if ($returnCode != 1) {

		printf ($LOG "Error: $Config::workDir/$rRcliper->getData() entered does not exist.\n");
		return -1;	#no input data file 
	
	}
	
	
	#================================================
	# *** Delete old grid/ps/nmapfiles 
	#================================================
	
	
	unlink("$Config::workDir/$rPlot->{gridFile}");
	#unlink("$Config::workDir/$rPlot->{nmapFile}");
	
	
	#================================================
	# *** Create Plots
	#================================================
	
	
	$type =~ tr/A-Z/a-z/;

	if ($type eq "ps") {
	
		createPlot($rPlot, $rRcliper, $type, $LOG, $rRcliper->getBoundaries(), $rRcliper->getExtent());
		unlink("$Config::workDir/$rPlot->{gridFile}");
	
	}
	elsif ($type eq "grid") {
		
		createPlot($rPlot, $rRcliper, $type, $LOG, $rPlot->{nmap_boundaries}, $rPlot->{nmap_extent});
	
	}
	else {
	
		printf ($LOG "Error: Incorrect type must be either type 'ps' or 'grid'.\n");
     		return -2;
	
	}
	
	return 1;

}


#--------------------------------------------------------
# Subroutine: createPlot	
#
# Purpose:  Uses a GEMPAK to convert a text file into a GEMPAK 
#			grid.
#
# Note:		ASCTOGEM adds additional functionality to gempak
#           which allows a user to convert a regularly spaced ascii file
#           of a specified format into a gempak grid.
#
#           The asctogem routine is not part of the gempak distribution.
#           The code needs to be placed in a known directory.
#
# Ascii file format:
#				
#				line 1: yymmdd/hh00 0 parameterName
#				line 2-n: latitude longitude parameterValue
#
#		**Note: Line 2-n is listed from the lower left to the upper
#			    right of the grid area.
#
#	example: ascii file format
#
#				030604/1200  0  rain
#				8.000 -89.000 0.000
#				8.000 -88.750 0.000
#				8.000 -88.500 0.000
#				8.000 -88.250 0.000 ....
#
# Input:	$_[0] - Plot object
#			$_[1] - Rcliper object
#			$_[2] - log file handle
#			$_[3] - Bounding boundries of the GEMPAK grid to create
#			$_[4] - Number of points in the x/y directions of the
#					GEMPAK grid to create.
#			
# Output:	1 - Success
#			-1 - Error creating shell script for GEMPAK commands
#-------------------------------------------------------------------
sub createPlot {


	#===================================
	# *** Set Variables / Definitions***
	#===================================


	my ($rPlot, $rRcliper, $type, $LOG, $boundries, $extent) = @_;

	my $returnCode;
	my $GEMPAK = new IO::File;
	my $shellScript = "gempak.sh";


	#=====================================
	# ***  Create file for Gempak routines ***
	#=====================================


	#--- Create a shell script for GEMPAK commands

	$returnCode = generalUtils::openFile($Config::workDir, $shellScript, "write", \$GEMPAK);
	
	if ($returnCode != 1) {
	
		printf ($LOG "Error ($returnCode): $Config::workDir/$shellScript could not be opened for writing.\n");
		
	}

	#--- First line points to shell and soure profile

	printf ($GEMPAK "#!$rPlot->{shell} \n");
#	printf ($GEMPAK ". $rPlot->{cronProfile} \n");


	#=====================================
	# ***  Gempak Commands ***
	#=====================================
	
	
	#--- Create a gempak grid from the ascii file
	
	asctogem($rPlot, $rRcliper, $GEMPAK, $boundries, $extent);
	
	if ($type eq "ps") {
	
		#--- Create a plot of the data with contours

		gdplot2($rPlot, $rRcliper, $GEMPAK, $boundries);

		#--- Create a plot of the a title

		gpmap($rPlot, $rRcliper, $GEMPAK, $boundries);

		#--- Puts storm track on the graphic in red with x's at the points

		atest($rRcliper, $GEMPAK);
	
	}
	
	#============================================
	# *** Close command file and execute code
	#============================================
	
	
	printf  ($GEMPAK "gpend\n");
	close ($GEMPAK);
	
	
	#=====================================
	# *** Execute GEMPAK code
	#=====================================
	

	chmod (0744, "$Config::workDir/$shellScript");
	system ("cd $Config::workDir; $Config::workDir/$shellScript");


	#=====================================
	# *** Clean up ***
	#=====================================
	

	unlink ("$Config::workDir/$shellScript");
	unlink ("$Config::workDir/gemglb.nts");
	unlink ("$Config::workDir/last.nts");
	return 1;
	

}

#--------------------------------------------------------
# Subroutine: addPlot
#
# Purpose:  Adds a new GEMPAK grid to an existing GEMPAK grid.
#		 	
# Input: $_[0] - Plot object representing the GEMPAK grid that will
#				 be added to an existing GEMPAK grid file
#		 $_[1] - The name of an existing GEMPAK grid file
#		 $_[2] - file handle to a log file for errors
#
# Output: 1 success
#		 -1 could not create shell script for GEMPAK commands
#--------------------------------------------------------
sub addPlot {


	#=====================================
	# *** Define local variables
	#=====================================

		
	my($rPlot, $file, $LOG) = @_;
	
	my $returnCode;
	my $shellScript = "temp.sh";
	my $GEMPAK = new IO::File;
	
	
	#==================================================
	# *** Create a shell script for GEMPAK commands
	#==================================================


	$returnCode = generalUtils::openFile($Config::workDir, $shellScript, "write", \$GEMPAK);
	
	if ($returnCode != 1) {
	
		printf ($LOG "Error ($returnCode): $Config::workDir/$shellScript could not be opened for writing.\n");
		return $returnCode;
		
	}
	
	#--- First line points to shell and soure profile

	printf ($GEMPAK "#!$rPlot->{shell} \n");
	#printf ($GEMPAK ". $rPlot->{cronProfile} \n");
	
	gdmod($rPlot, $file, $LOG, $GEMPAK);
	
	close $GEMPAK;
	
	
	#=====================================
	# *** Execute script to add grids
	#=====================================
	
	
	chmod (0744, "$Config::workDir/$shellScript");
	system ("$Config::workDir/$shellScript");
	
	
	#=====================================
	#*** Clean up
	#=====================================
	
	
	unlink ("$Config::workDir/$shellScript");
	
	return 1;
}

#--------------------------------------------------------
# Subroutine: asctogem		
#
# Purpose:  This includes additional functionality to gempak
#           which allows a user to convert a regularly spaced ascii file
#           of a specified format into a gempak grid.
#
#           The asctogem routine is not part of the gempak distribution.
#           The code needs to be placed in a known directory.
#
#	Ascii file format:
#				
#				line 1: yymmdd/hh00 0 parameterName
#				line 2-n: latitude longitude parameterValue
#
#		**Note: Line 2-n is listed from the lower left to the upper
#			    right of the grid area.
#
#	example: ascii file format
#
#				030604/1200  0  rain
#				8.000 -89.000 0.000
#				8.000 -88.750 0.000
#				8.000 -88.500 0.000
#				8.000 -88.250 0.000 ....
#
#--------------------------------------------------------
sub asctogem {

	#--- File Handles
	
	my ($rPlot, $rRcliper, $GEMPAK, $boundaries, $extent) = @_;
	my $datafile = $rRcliper->getData(); 

print $GEMPAK <<END_OF_TEXT

$rPlot->{asctogem}<<EOF ;
$Config::workDir/$datafile
$rPlot->{gridFile}
$rPlot->{proj}  
$boundaries
$extent
$rPlot->{totalgrids}
$rPlot->{packing}
$rPlot->{ivcord}
$rPlot->{packing}

EOF

END_OF_TEXT

}

#--------------------------------------------------------
# Subroutine: gdplot2	
#
# Purpose: Plots the data in the gempak grid by contouring
#		   it and filling in the contours with specified colors.
#
#	GDFILE    Grid file                         $GEMDATA/HRCBOB.GRD
# 	GDATTIM   Grid date/time                    LAST
# 	GLEVEL    Grid level                        500
# 	GVCORD    Grid vertical coordinate          PRES
# 	PANEL     Panel loc/color/dash/width/regn   0
# 	SKIP      Skip_cntr/skip_plt_x;skip_plt_y   0
# 	SCALE     Scalar scale / vector scale       999
# 	GDPFUN    Scalar grid or vector grid funct  TMPC
# 	TYPE      GDPLOT2 function processing type  C
# 	CONTUR    Subbox/smooth                     0
# 	CINT      Contour interval/min/max          0
# 	LINE      Color/type/width/label/smth/fltr  3
# 	FINT      Fill interval/min/max             0
# 	FLINE     Fill colors/fill types            10-20
# 	HILO      Color/symbol/rng/rad/cnt/intp
# 	HLSYM     HILO txt size/posn/font/wdth/hw
# 	CLRBAR    Color/ornt/anch/x;y/ln;wd/freq|t
# 	WIND      Wind symbol/siz/wdth/typ/hdsz     BM1
# 	REFVEC    Mag;x;y;txtsiz/font/wdth/HW;labl
# 	TITLE     Title color/line/title            1
# 	TEXT      Size/fnt/wdth/brdr/N-rot/just/hw  1
# 	CLEAR     Clear screen flag                 YES
# 	GAREA     Graphics area                     WV
# 	PROJ      Map projection/angles/margins|dr  MER
# 	MAP       Map color/dash/width/filter flag  1
# 	LATLON    Line color/dash/width/freq/inc/l
# 	DEVICE    Device|name|x size;y size|color   XW
# 	STNPLT    Txtc/txt attr|marker attr|stnfil
# 	SATFIL    Satellite image filename(s)
# 	RADFIL    Radar image filename(s)
# 	LUTFIL    Enhancement lookup table filenam
# 	STREAM    lines/arrows/stop/slow/scale
# 	POSN      Position                          0
# 	COLORS    Color list                        1
# 	MARKER    Marker color/type/size/width/hw   0
# 	GRDLBL    Grid point label color            0
# 	FILTER    Filter data factor                YES
#--------------------------------------------------------
sub gdplot2 {

	#--- File Handles
	
	my ($rPlot, $rRcliper, $GEMPAK, $boundaries) = @_;
	
print $GEMPAK <<END_OF_TEXT

gdplot2 <<EOF3 ;
GDFILE   = $Config::workDir/$rPlot->{gridFile} 
GDATTIM  = LAST
GLEVEL   = 0
GVCORD   = none
PANEL    = 0
SKIP     = 0
SCALE    = 0
GDPFUN   = mul($$rPlot{parameter}, 1.0)
TYPE     = C
CONTUR   = 0
CINT     = 1/0.5/0.5
LINE     = 30/5/1/0
FINT     = 
FLINE    = 
HILO     =  
HLSYM    =  
CLRBAR   = 0  
WIND     =  
REFVEC   =  
TITLE    = 0 
TEXT     = 1.2/22//221///hw 
CLEAR    = YES
GAREA    = $boundaries 
PROJ     = $rPlot->{proj} 
MAP      = 1//4
LATLON   = 0 
DEVICE   = $rPlot->{device} | $rPlot->{psFile}
STNPLT   =  
SATFIL   =  
RADFIL   =  
LUTFIL   =  
STREAM   =  
POSN     = 0
COLORS   = 1
MARKER   = 0
GRDLBL   = 0
FILTER   = YES

run

EOF3

gdplot2 <<EOF2 ;
GDFILE   = $Config::workDir/$rPlot->{gridFile} 
GDATTIM  = LAST
GLEVEL   = 0
GVCORD   = none
PANEL    = 0
SKIP     = 0
SCALE    = 0
GDPFUN   = mul($$rPlot{parameter}, 1.0)
TYPE     = C/F
CONTUR   = 0
CINT     = 1/1/100
LINE     = 30/1/3/2
FINT     = 1/1/100
FLINE    = 0;27;25;24;23;22;21;20;19;18;17;16;15;14 
HILO     =  
HLSYM    =  
CLRBAR   = 1  
WIND     =  
REFVEC   =  
TITLE    = 0 
TEXT     = 1.2/22//221///hw 
CLEAR    = NO
GAREA    = $boundaries 
PROJ     = $rPlot->{proj} 
MAP      = 1//4
LATLON   = 0 
DEVICE   = $rPlot->{device} | $rPlot->{psFile}
STNPLT   =  
SATFIL   =  
RADFIL   =  
LUTFIL   =  
STREAM   =  
POSN     = 0
COLORS   = 1
MARKER   = 0
GRDLBL   = 0
FILTER   = YES

run

EOF2

END_OF_TEXT

}

#--------------------------------------------------------
# Subroutine: gpmap		
#
# Purpose: Adds the title to the plot.
#
#	MAP       Map color/dash/width/filter flag  1
# 	GAREA     Graphics area                     WV
# 	PROJ      Map projection/angles/margins|dr  MER
# 	SATFIL    Satellite image filename(s)
# 	RADFIL    Radar image filename(s)
# 	LATLON    Line color/dash/width/freq/inc/l
# 	PANEL     Panel loc/color/dash/width/regn   0
# 	TITLE     Title color/line/title            1
# 	TEXT      Size/fnt/wdth/brdr/N-rot/just/hw  1
# 	CLEAR     Clear screen flag                 YES
# 	DEVICE    Device|name|x size;y size|color   XW
# 	LUTFIL    Enhancement lookup table filenam
# 	STNPLT    Txtc/txt attr|marker attr|stnfil
# 	VGFILE    Vgfile | scale file | attribute
# 	AFOSFL    AFOS Graphics File
# 	AWPSFL    AWIPS Graphics File
# 	LINE      Color/type/width/label/smth/fltr  3
# 	WATCH     End time|Ts clr;Tn clr;St clr|Tm
# 	WARN      End time|TS;TN;FF clrs|Tm|Lb
#	HRCN      End time|colors|syms|Tm|Lb|Mt|Qw
# 	ISIG      End time|colors|Sym|Tm|Id|Mv|Fl
# 	LTNG      End time|time ints/colors|marker
#	ATCF      Time|colors|models|Tm|Id|Mv|Mkr|
# 	AIRM      End time|colors|Sym|Tm|Fl
# 	NCON      End time|colors|Sym|Tm|Id|Fl
#	SVRL      End time|TS;TN clrs|Tm|Lb
# 	BND       Bnd name/color/fillsiz/fillpat/f
# 	ATCO      Time|colors|models|Tm|Id|Mv|Mkr|
#	TCMG      End time|colors|center
#	 QSCT      End time|spd ints/colrs|bsiz;bwi
#
#--------------------------------------------------------
sub gpmap {

	#--- File Handles
	
	my ($rPlot, $rRcliper, $GEMPAK, $boundaries) = @_;
	
print $GEMPAK <<END_OF_TEXT

gpmap <<EOF3 ;
MAP      = 1//3
GAREA    = $boundaries 
PROJ     = $rPlot->{proj} 
SATFIL   =
RADFIL   =
LATLON   = 1/10/3/1;1/5;5//2
PANEL    = 0
TITLE    = 1/-2/$rPlot->{title}
TEXT     = 1.5/22//221///hw
CLEAR    = no 
DEVICE   = $rPlot->{device} | $rPlot->{psFile}
LUTFIL   =
STNPLT   =
VGFILE   = 

li

run

exit
EOF3


END_OF_TEXT

}


#--------------------------------------------------------
# Subroutine: atest	
#
# Purpose: Add the track points to the plot in red with x's at the points
#
#--------------------------------------------------------

sub atest {

	#--- File Handles
	
	my ($rRcliper, $GEMPAK) = @_;
	my $trkPoints = $rRcliper->getTrack();
	my $trklines = @$trkPoints;
	my $i;
	
print $GEMPAK <<END_OF_TEXT ;	

atest <<EOF4 ;
ginitp
0 
gsline
1 0 3 0
gscolr
2
gline
m
$trklines
END_OF_TEXT
  
foreach my $point (@$trkPoints) {

	print $GEMPAK "$point \n";
	
}

print $GEMPAK <<END_OF_TEXT2;
geplot
gmark
m
$trklines
END_OF_TEXT2

foreach my $point (@$trkPoints) {

	print $GEMPAK "$point \n";
	print $GEMPAK "gsmrkr\n";
	print $GEMPAK "20 0 2 3\n";
	print $GEMPAK "gscolr\n";
	print $GEMPAK "1\n";
	
}

print $GEMPAK <<END_OF_TEXT3;
geplot
exit

EOF4

END_OF_TEXT3

}


#--------------------------------------------------------
# Subroutine: gdmod	
#
# Purpose: moves grids from one GEMPAK grid file to another
#
#	GDFILE   Grid file
#	GDOUTF   Output grid file
# 	GDATTIM  Grid date/time
#	GLEVEL   Grid level
#	GFUNC    Scalar grid
#	GVCORD   Grid vertical coordinate
#	GPACK    Packing type/number of bits
#	GDLIST lists data from a scalar grid.
#	GAREA    Graphics area
#	PROJ     Map projection/angles/margins|drop flag
#	SCALE    Scalar scale / vector scale
#	OUTPUT   Output device/filename
#
#
# Inputs: $_[0] - absolute path 
#	  $_[1] - new grid file
#	  $_[2] - output grid file
#	  $_[3] - parameter to add
#
#--------------------------------------------------------
sub gdmod {

	#--- Define local variables
		
	my($rPlot, $file, $LOG, $GEMPAK) = @_;
	my $returnCode;

	#--- First line points to shell

	printf ($GEMPAK "#!$rPlot->{shell} \n");

	#--- Source profile

	#printf ($GEMPAK "$rPlot->{cronProfile} \n");
	
	#--- Create a gempak grid from the ascii file
	
print $GEMPAK <<END_OF_TEXT ;

gdmod <<EOF;
GDFILE   = $Config::workDir/$rPlot->{gridFile}
GDOUTF   = $Config::workDir/$file
GDATTIM = LAST
GLEVEL  = 0
GFUNC   = $$rPlot{parameter}
GVCORD  = NONE
GPACK   = 

run

EOF

END_OF_TEXT
	
	#--- Execute commands written to the text file

	printf $GEMPAK "gpend \n";
	
}


#--------------------------------
#
#	Accessor methods
#
#--------------------------------


#------------------------------------------------
# Subroutine: getNmapName
# 
# Purpose: Return the name of the NMAP grid file
#			to the caller.
#----------------------------------------------
sub getNmapName {

	my ($rPlot) = @_;
	
	return $rPlot->{nmapFile};

}

#------------------------------------------------
# Subroutine: getGridName
# 
# Purpose: Return the name of the grid file
#			to the caller.
#----------------------------------------------
sub getGridName {

	my ($rPlot) = @_;
	
	return $rPlot->{gridFile};

}

#------------------------------------------------
# Subroutine: getPSname
# 
# Purpose: Return the name of the Postscript file
#			to the caller.
#----------------------------------------------
sub getPSname {

	my ($rPlot) = @_;
	
	return $rPlot->{psFile};

}
