package btk_data;

use File::Copy;
use strict;

#my $rootDir = "/home/devel/jht";
#require "$rootDir/generalCode/generalUtils_v1.pl";

#=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Program Name: btk_data.pl
#
#
# Language: Perl 5.8.x
#
#
# Purpose: Takes a storm name an finds the corresponding btk file.
#			The btk file is parsed and placed into a btk object.
#
#
# Input: storm name in bb##YYYY format
#
#
# Output: 	
#
#
# Requires: generalUtils.pl - includes general file routines
#			File::Copy
#    
# Called by: rcliper_v6.pl
#----------------------------------------------------------------------
# Data Structure:  
#----------------------------------------------------------------------
## Subroutines: 
#
#----------------------------------------------------------------------
# Information:
#    Joint Hurricane Testbed/USWRP project
#    National Hurricane Center / Tropical Prediction Center
#    Department of Commerce - NOAA / NWS
#
# Category: JHT - RI Plotting
#
# Written by: Alison Krautkramer 09/2003
#
#Modified: 3/2004 
#		Prepared code for transfer to operations
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=




#==============================
# *** Global Variables ***
#==============================




#--- Hash used to index into the best track object

my %objIndex = (
						"wind" 	=> 0,
						"intensity" => 1,
						"rmw"		=> 2,
						"latitude"	=> 3,
						"longitude" => 4
					
			   );




#==============================
# *** Object Initilizaion Method ***
#==============================




#--------------------------------------------------------
# Subroutine: new
#
# Purpose:  
#--------------------------------------------------------------
sub new {

	my ($storm, $workDir, $btkDir, $LOG) = @_;
	my $returnCode;
	
	#--- Make $storm all upper case

	$storm =~ tr/A-Z/a-z/;	

	#--- hash of data

	my %hData = ( "hValues"    => {},
		      "file"	   => "b" . $storm . ".dat"
                );
		
	$returnCode = getData(\%hData, $workDir, $btkDir, $LOG);
	
	if ($returnCode != 1) {
	
		printf ($LOG "Error ($returnCode): from btk:getdata.\n");
		return undef;
		
	}
				
	return \%hData;
	
}




#======================================
# *** GET DATA from file Methods ***
#======================================


#--------------------------------------------------------
# Subroutine: getData
#
# Purpose: Copies a best track atcf file to the working directory.  The
#		   file is opened and read.  The relevant values are placed in a 
#		   hash.
#
# Called by: main
#
# Calls: extractData() - gets best track values from best track file
#
# Inputs: $_[0] - Best track object 
#	  $_[1] - directory containing the best track ATCF files
#	  $_[1] - work directory
#	  $_[2] - name of the ships file
#		
# Outputs: 1 - success
#		 Filled ($_0) RI object with information read from SHIPS file
#
# Errors:  -1 file does not exist
#	   -2 error opening file
#	   -4 file is not readable
#	   -5 file has with zero size
#	   -6 Error copying file
#	   -7 Format Error
#	   -8 One of the RI values is not defined
#--------------------------------------------------------
sub getData {


	#====================================
	#	*** Define Variables ***
	#===================================
	
	
	my ($rhData, $workPath, $dataPath, $LOG) = @_;
	
	my $INPUT = new IO::File;
	my $line;
	my $returnCode;
	my @result;
	
	my $dtg = undef;
	my $wind = undef;
	my $intensity = undef;
	my $rmw = undef;
	my $lat = undef;
	my $long = undef;

	
	#====================================
	#	*** Copy/Open the Best Track file ***
	#===================================
	
	
	#--- Copy file to work directory
	copy ("$dataPath/$$rhData{file}", "$workPath/$$rhData{file}") || return -6;
	
	#--- Open file
	
	$returnCode = generalUtils::openFile($workPath, $$rhData{file}, "read", \$INPUT);
	
	if ($returnCode != 1) {
	
		print ($LOG "Error ($returnCode): Opening $workPath/$$rhData{file}.\n");
		return $returnCode;
		
	}
	
	
	#===============================================
	#	*** Extract Data from the Best Track file ***
	#================================================
	
	
	while (defined ($line = <$INPUT>)) {
	
		#--- Initialize result
		
		@result = ();
	
		#--- remove end of line character
		
		chomp $line;
		
		#--- split the line between commas
		
		@result = split(/\,/, $line);
		
		if (@result <= 0) {
			
				next;	#this line is blank go to next line
				
		}
		
		#--- Remove all spaces
		
		foreach my $element (@result) {
		
			$element =~ s/\s+//g;
		
		}
		
		#--- Check that the results one or more numerical digits
		#--- Check that the dtg element's format is valid
		
		if (defined($result[2]) && ($result[2] =~ /[0-9]+/) && (generalUtils::validDateFormat($result[2]) == 1)) {
			
				$dtg = $result[2];
				
				if (defined($result[8]) && $result[8] =~ /[0-9]+/) {
					
					$wind = $result[8]
				
				}
				
				if (defined($result[9]) && $result[9] =~ /[0-9]+/) {
				
					$intensity = $result[9];
				
				}
				
				if (defined($result[19]) && $result[19] =~ /[0-9]+/) {
				
					$rmw = $result[19];
				
				}
				
				if (defined($result[7]) && $result[7] =~ /[0-9]+/) {
				
					$long = $result[7];
				
				}
				
				if (defined($result[6]) && $result[6] =~ /[0-9]+/) {
				
					$lat = $result[6];
				
				}
				
				
		}
		else {
		
			return -1;	#Error - format problem with the data/time group
			
		}
	
	
		#======================================================
		#	*** Initalize data structure using the dtg
		#======================================================
	
		
		if (! (defined ($$rhData{hValues}->{$dtg}))) {
		
			
			for (my $i = 0; $i < keys(%objIndex); $i++) {
			
				$$rhData{hValues}->{$dtg}[$i] = undef; 
				
			}
			
		}
		
	
		#======================================================
		#	*** Add results to data structure
		#======================================================
	
		
		$$rhData{hValues}->{$dtg}[$objIndex{intensity}] = $intensity;
		$$rhData{hValues}->{$dtg}[$objIndex{wind}] = $wind;
		$$rhData{hValues}->{$dtg}[$objIndex{rmw}] = $rmw;
		$$rhData{hValues}->{$dtg}[$objIndex{latitude}] = $lat;
		$$rhData{hValues}->{$dtg}[$objIndex{longitude}] = $long;
	
		#===================================================================
		# *** Return Success if either $intensity or $maxWind 
		#===================================================================
	
	
		if ((!defined($intensity)) && (!defined($wind))) {
		
			return -2;	#no values read from file	
		
		}
	}
	
	close $INPUT;
	
	return 1;
	
}



#--------------------------------------------------------
# Subroutine: mergeBtkData (normalized) ... Incomplete....
#
# Purpose: 
#
# Called by: 
#
# Outputs: 
# 
#--------------------------------------------------------
sub mergeBtkData {
	
	my ($rhData, $raValues, $type) = @_;
	
	my $num = @{$raValues};
	my $count = 0;
	
	#check for a valid $type definition
	
	if (! defined($type) || ! defined($objIndex{$type})) {
	
		return -1; #Error incorrect type or type not found
	
	}
	
	foreach my $DTG (@{$$raValues[0]}) {
	
		if (defined ($$rhData{hValues}->{$DTG})) {
		
			#Normalize values for printing
			#intensity
			#my $normal = ((5 * ($$rhData{hValues}->{$DTG}->[$objIndex{$type}] - 880)) / (1100 - 880));
			#$$raValues[$num][$count] = $normal;
			
			$$raValues[$num][$count] = $$rhData{hValues}->{$DTG}->[$objIndex{$type}];
			
		}
		else {
		
			$$raValues[$num][$count] = undef;
		
		}
		
		
		$count++;
	
	}
	
	return 1;
	
}


#======================================
# *** Reformat Data for plotting ***
#======================================




#--------------------------------------------------------
# Subroutine: reformat
#
# Purpose: Takes a hash and reformats the data values for plotting.
#
#		   To reformat the hash:
#				1. Place the hash keys into an array.  This array
#				   is then sorted by dtg in ascending order 
#				2. Go through the array and add in any missing values 
#			    3. The sorted array of keys are used to access 
#				   the hash in an ordered way.  
#
# Inputs: $_[0] - hash
#
#				The input hash uses the AMSU file names as keys
#				to arrays 
#	
#				Example ( {2003061712}, ([undef,998,1001], [undef, 30, 20]),
#						  {2003061718}, ([990,undef,1001], [undef, 20, 30]),
#						   .... )
#
# Output: 2 dimensional array of intensities
#
#				The product of this function are two 2-Dimensional
#				arrays which contain linked
#				dtgs and AMSU values in ascending dtg order.
#
#				Example ( [2003061712, 2003061718],
#						  [undef,990]
#						  [998, undef]
#						  [1001,1001]
#					    ) 	
#
# Called by: 
#
# Calls: by_dtg - to sort the data into ascending dtg
#
# Errors: 
#--------------------------------------------------------
sub reformat {


	my ($rhData, $raValues, $type) = @_;
	
	my @aSorted = ();			# array to store ordered DTG's
	my @aCompleteDTG = ();		# array to store a continuous array of DTG's	
	my $value = 0; 				# keeps track of whether the array has any valid values	
	
	
	#=============================
	#	*** Sort DTG's ***
	#=============================
	
	
	#----------------------------------------------
	# Sort the keys in the hash (ie DTG) in ascending
	# order.  Place the sorted list of DTG's in an 
	# array so that chart results are sorted.
	#-----------------------------------------------
	
	
	@aSorted = sort by_dtg keys(%{$$rhData{hValues}});
	
	#---Make sure the Complele DTG array starts with some date at 0z
	
	my $firstDTG = $aSorted[0];
	
	if (substr($aSorted[0], 8,2) ne "00") {
	
		my $firstDTG = substr($aSorted[0], 0, 8) . "00";
		unshift(@aSorted, $firstDTG);
		
	}
	
	#continuousDTG(\@aSorted, \@aCompleteDTG);
	generalUtils::continuousDTG(\@aSorted, \@aCompleteDTG, 6);
	
	
	#==================================
	#	*** Reformat Hash into a 2-D array ***
	#==================================
	
	
	#--------------------------------------------------------------------------
	# Reformat the hash into a 2-D array that can be plotted by GD graph.
	# Since new values were added to the sorted array so that every hour
	# would be represented by a tick mark on the graph, many of the entries
	# in the complete sorted array are not defined in the associated hash function.
	# So two cases can occur
	#			1. the DTG in the complete array is represented in the 
	#			   hash and the value is copied directly to the 2-D array
	#			2. the DTG in the complete array is NOT represented in the
	#			   hash so just enter an undefined value for each of the satellites.
	#-----------------------------------------------------------------------------
	
	for (my $i = 0; $i < @aCompleteDTG; $i++) {
	
		my $DTG = $aCompleteDTG[$i];
		
		
		#==================================
		#	*** Initialize 2 dimensional array ***
		#==================================
	
		$$raValues[0][$i] = $DTG;
		
		#--- If this is a new DTG then mark the data as undef
		
		if (!defined ($$rhData{hValues}->{$DTG})) {
	
			foreach my $element (keys %objIndex) {
				
				$$raValues[$objIndex{$element} + 1][$i] = undef;
				
			}
			
		}
		
		#--- If this DTG has already been seen in the CIRA files then 
		#--- copy the data that has been read into the array
		
		else {
				
			if (defined($type) && defined($objIndex{$type})) {
					
						
				if (defined ($$rhData{hValues}->{$DTG}[$objIndex{$type}])) {
						
					$$raValues[$objIndex{$type} + 1][$i] = $$rhData{hValues}->{$DTG}[$objIndex{$type}];
					$value = 1;
					
				}
				else {
						
					$$raValues[$objIndex{$type} + 1][$i] = undef;
						
				}
						
			}
		}
	
	}
	
	if ($value == 1) {
	
		return 1; 			#success there is some defined data to plot
							#that is none zero
	
	}
	else {
	
		$raValues= undef;
		return -1;			#no valid data to plot
	
	}
		

}

#--------------------------------------------------------
# Subroutine:reformatDTG
#
# Purpose: Reformats an array of date/times for pretty plotting.
#			
#			Rule: If the time group is 0Z then plot entire
#				  date/time string (ex 2003092600)
#
#				  if the time group does not equal 0Z then only
#				  plot the time group (ex 06)
#
# Input: $_[0] A reference to an array of dtg values
#
# Outputs: A reference to an array of modified dtg values based on
#			the rule listed above.
# 
#--------------------------------------------------------
sub reformatDTG {

	my ($raValues) = @_;
	
	for (my $i = 0; $i < @{$$raValues[0]}; $i++) {
	
		my $DTG = $$raValues[0][$i];
		
		if (substr($DTG, 8, 2) eq "00") {
	
			$$raValues[0][$i] = substr($DTG, 4, 6);
		
		}
		else {
		
			$$raValues[0][$i] = substr($DTG, 8, 2);
		
		}
	
	}
	

}
#--------------------------------------------------------
# Subroutine: continuousDTG
#
# Purpose: Sorting routine used by the createPlots subroutine
#		   to arrange the AMSU text files in ascending order
#		   using the dtg within the files title.
#
# Called by: createPlots
#
# Inputs:	$_[0] - Reference to an array of sorted DTG's
#			$_[1] - Reference to an empty array
#
# Outputs: The missing date/time groups from the sorted array
#			are calculated and inserted in the new array.  As
#			a result the calling routine will get back an array'
#			of continuouly spaced date/time groups.
#
#			example input: {20030912600, 2003090818}
#					output: {20030912600, 2003090806, 2003090812, 2003090818}
#
# Returns: -1 $a comes before $b
#		    0 does not matter
#			1 $b comes before $a
# 
#--------------------------------------------------------
sub continuousDTG {


    	#=================================================
    	# *** Define local variables ***
    	#=================================================


	my ($raSortedDTG, $raCompleteDTG) = @_;
	my $nextDTG;
	
	#==================================
	#	*** Add missing DTG's ***
	#==================================
	
	
	#----------------------------------------------
	# Since the Date/time of an AMSU satellite pass can 
 	# occur at any time of the day.  The GD module needs the 
	# data ploted on the x axis to be regularly spaced.  Therefore,
	# the Date/time of the AMSU satellite pass is rounded to the 
	# nearest hour and all hour are represented on the graph.
	#-------------------------------------------------------
	
	$$raCompleteDTG[0] = $$raSortedDTG[0];
	$nextDTG = generalUtils::nextTime($$raSortedDTG[0], 6);
	
	for (my $i  = 0; $i < (@{$raSortedDTG} - 1); $i++) {
		
		my $found = 0;
		my $break = 0;
		
		while (($found == 0) && ($break < 40)) {
			
		
			#--- The next Date time is not found in the sorted array
			#--- so add the value to the complete sorted Array
		
			if ($$raSortedDTG[$i+1] != $nextDTG) {
			
				$$raCompleteDTG[@{$raCompleteDTG}] = $nextDTG;
				
			}
			
			#--- The next Date time was found in the sorted array
			#--- so enter the value in the complete sorted array
			#--- and move to the next Date time stored in the sorted array
			
			else {
			
				$$raCompleteDTG[@{$raCompleteDTG}] = $$raSortedDTG[$i + 1];
				$found = 1;
				
			}
			
			#--- Get the time + 1 hour
			
			$nextDTG = generalUtils::nextTime($nextDTG, 6);
			$break++;
			
		}
		
	}

}

#--------------------------------------------------------
# Subroutine: by_dtg
#
# Purpose: Sorting routine used by the createPlots subroutine
#		   to arrange the AMSU text files in ascending order
#		   using the dtg within the files title.
#
# Called by: createPlots
#
# Outputs: -1 $a comes before $b
#		    0 does not matter
#			1 $b comes before $a
# 
#--------------------------------------------------------
sub by_dtg {
	$a <=> $b;
}


#======================================
# *** Accessor Methods ***
#======================================

#--------------------------------------------------------
# Subroutine: getIntensity
#
# Purpose: Returns a hash of dtg pointing to a hash of btk values
#
# Inputs: $_[0] - reference to a hash containing btk values
#	  $_[1] - reference to a hash containing btk values
#
# Outputs: none
#
#--------------------------------------------------------
sub getIntensity {

    #===========================
    # *** Define Variables
    #===========================
	
	
	my ($rhData, $hValues, $aSorted) = @_;
	my $dtg;
	
	#--- Define an array to access the hash in an ordered way
	
	@$aSorted = sort by_dtg keys(%{$$rhData{hValues}});
    
	foreach $dtg (keys(%{$$rhData{hValues}})) {
		
		$$hValues{$dtg}->{intensity} = $$rhData{hValues}->{$dtg}->[$objIndex{wind}];
		$$hValues{$dtg}->{latitude} = $$rhData{hValues}->{$dtg}->[$objIndex{latitude}];
		$$hValues{$dtg}->{longitude} = $$rhData{hValues}->{$dtg}->[$objIndex{longitude}];
			
	}


}


#--------------------------------------------------------
# Subroutine: printResults
#
# Purpose: Prints the SHIPS data values stored in the RI object
#		   to the passed in file handle
#
# Inputs: $_[0] - reference to a hash containing SHIPS values
#		  $_[1] - handle to an open file
#
# Outputs: none
#
#--------------------------------------------------------
sub printResults {


	#===========================
    	# *** Define Variables
    	#===========================
	
	
	my ($rhData, $FH) = @_;
    	my $key;
	

	#================================
    	# *** Print to file handle $FH
	#================================

	
	#--- Print to file handle $FH
	
	printf ($FH  "DTG   Maxwind Intensity  RMW\n");
	
	foreach $key (keys(%{$$rhData{hValues}})) {
		
		print ($FH "$key ");
			
		if (defined($$rhData{hValues}->{$key}->[0])) {
		
			print ($FH "$$rhData{hValues}->{$key}->[0] ");
		
		}
		else {
		
			print ($FH "-1 ");
		
		}
			
		if (defined($$rhData{hValues}->{$key}->[1])) {
		
			print ($FH " $$rhData{hValues}->{$key}->[1]");
		
		}
		else {
		
			print ($FH " -1");
		
		}
		
		if (defined($$rhData{hValues}->{$key}->[2])) {
		
			print ($FH " $$rhData{hValues}->{$key}->[2]\n");
		
		}
		else {
		
			print ($FH " -1\n");
		
		}
			
	}

}

1;
