package btkData;

use File::Copy;
use strict;
use IO::File;

require "generalUtils_v1.pl";

#=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
# Program Name: btk_data.pl
#
# Version: 1
#
# Language: Perl 5.8.x
#
#
# Purpose: Takes a storm name an finds the corresponding btk file.
#			The btk file is parsed and placed into a btk object.
#			This data can then reformated for plotting.
#
#
# Inputs:	$storm (string) - string containing a unique storm id (ex - al132003)
#			$workDir (string) - Directory for temp work files
#			$btkDir (string) - Directory containing the ATCF Best Track Files
#			$LOG (ref file handle) - file handle for output
#
# --------------- SUBROUTINES--------------------------
#
## Subroutine: new
#
# Purpose:  Creates a data structure to store information read
#			from the Best Track file associated with the given storm
#
## Subroutine: getData
#
# Purpose: Copies a best track atcf file to the working directory.  The
#		   file is opened and read.  The relevant values are placed in a 
#		   hash.
#
## Subroutine: checkParam
#
# Purpose: Checks if a number parsed from the best track file is defined
#			and contains 1 or more digits.  If these checks are successful
#			the best track value is added to the data structure.
#
## Subroutine: mergeBtkData
#
# Purpose: Merge best track data from a best track data object into
#			a 2 dimensional array where the first row of the array
#			contains dtg's in the format YYYYMMDDHH
#
## Subroutine: reformat
#
# Purpose: Takes a hash and reformats the data values for plotting.
#
#		   To reformat the hash:
#				1. Place the hash keys into an array.  This array
#				   is then sorted by dtg in ascending order 
#				2. Go through the array and add in any missing values 
#			    3. The sorted array of keys are used to access 
#				   the hash in an ordered way.  
#
## Subroutine:: reformatDTG
#
# Purpose: Reformats an array of date/times for pretty plotting.
#			
#			Rule: If the time group is 0Z then plot entire
#				  date/time string (ex 2003092600)
#
#				  if the time group does not equal 0Z then only
#				  plot the time group (ex 06)
#
## Subroutine: by_dtg
#
# Purpose: Sorting routine used by the createPlots subroutine
#		   to arrange the AMSU text files in ascending order
#		   using the dtg within the files title.
#
## Subroutine: printResults
#
# Purpose: Prints the SHIPS data values stored in the RI object
#		   to the passed in file handle
#
#
#----------------EXTERNAL PROGRAMS-------------------
#
# Requires: generalUtils.pl - includes general file routines
#    
# Called by: rcliper_v6.pl, ri_v4.pl
#
#----------------DATA STRUCTURE ---------------------
#
# my %hData = ( 
#					"hValues"    => {},
#				  	"file"	   => "b" . $storm . ".dat"     
#			  );
#
# 	hValues - A hash to store Best Track data values.  The key
#			  of the hash is the DTG of the Best Track value.  This
#			  key uniquely identifies an array of Best Track values.
#				key -> dtg
#				store -> array of best track data with the following order
#					    [wind, intensity, rmw, latitude, longitue]
#
#   file - name of the best track file that was used to populate the obj
#
# my %objIndex = (
#					"wind" 	=> 0,
#					"intensity" => 1,
#					"rmw"		=> 2,
#					"latitude"	=> 3,
#					"longitude" => 4
#			     );
#
#	The %objIndex links the key words which uniquely specify categories
#	of best track data with their placement in the array stored in the 
#	%hData object.
#
#	Example: $hData{hValues}->{2003040500} => [13, 30, 40, 60, 70]
#
#			in this example wind = 13
#							intensity = 30
#							rmw = 40
#							latitude = 60
#							longitude = 70
#
#------------------------INFORMATION-----------------------------------
#
# Joint Hurricane Testbed/USWRP project
# National Hurricane Center / Tropical Prediction Center
# Department of Commerce - NOAA / NWS
#
# Category: JHT - RI Plotting
#
# Written by: Alison Krautkramer 09/2003
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=




#==============================
# *** Global Variables ***
#==============================




#--- Hash used to index into the best track object

my %objIndex = (
						"wind" 	=> 0,
						"intensity" => 1,
						"rmw"		=> 2,
						"latitude"	=> 3,
						"longitude" => 4
					
			   );




#===========================================
# *** Object Initilizaion Method ***
#===========================================




#--------------------------------------------------------
# Subroutine: new
#
# Purpose:  Creates a data structure to store information read
#			from the Best Track file associated with the given storm
#
# Inputs:	$storm - (string)	 - unique storm id (ex - al132003)
#			$workDir - (string)  - Directory for temp work files
#			$btkDir - (string) 	 - Directory containing the ATCF Best Track Files
#			$LOG - (ref file handle)  - file handle for output
#
# Calls: getData() - read best track file and extract data for storage in 
#					 best track data object
#
# Output:	undef - error occured
#			reference to a data structure - success
#--------------------------------------------------------------
sub new {

	my ($storm, $workDir, $btkDir, $LOG) = @_;


	#=================================
	#	*** Check input parameters
	#=================================

	
	#--- Redirect $LOG to STDOUT if it is not defined by the caller
	
	if (!defined $LOG) {
	
		$LOG = new IO::File;
		$LOG->open(">&STDOUT");
	
	}
	
	#--- Check input variables
	
	if (!defined $storm || ! defined $workDir || ! defined $btkDir) {
	
		print ($LOG "Error(btkData::new): Input parameter not defined.\n");
		return -1;
	
	}
	
	#--- Make $storm all upper case

	$storm =~ tr/A-Z/a-z/;	


	#==================================
	#	***  Create Data Structure
	#==================================


	my %hData = ( 
				
					"hValues"    => {},
				  	"file"	   => "b" . $storm . ".dat"
                
				);
	
				
	#================================================
	#	*** Read Best Track File and Extract Values
	#================================================
	
		
	my $returnCode = getData(\%hData, $workDir, $btkDir, $LOG);
	
	if ($returnCode != 1) {
	
		printf ($LOG "Error(btkData::new): Extracting Best Track Values - returnCode $returnCode.\n");
		return undef;
		
	}
				
	return \%hData;
	
}




#======================================
# *** Data Methods ***
#======================================


#--------------------------------------------------------
# Subroutine: getData
#
# Purpose: Copies a best track atcf file to the working directory.  The
#		   file is opened and read.  The relevant values are placed in a 
#		   hash.
#
# Called by: new
#
# Calls: extractData() - gets best track values from best track file
#
# Inputs: $rhData - (ref data object) - Best track object 
#		  $workDir - (string) - work directory
#		  $btkDir - (string) - directory containing the best track ATCF files
#		  $LOG - (ref file handle) - file handle for redirected I/O
#		
# Outputs: 1 - success
#		   Filled RI object with information read from SHIPS file
#
# Errors:  -1 file does not exist
#		   -2 error opening file
#		   -4 file is not readable
#		   -5 file has with zero size
#		   -6 Error copying file
#		   -7 Format Error
#		   -8 One of the RI values is not defined
#--------------------------------------------------------
sub getData {


	#====================================
	#	*** Define Variables ***
	#===================================
	
	
	my ($rhData, $workDir, $btkDir, $LOG) = @_;
	
	my $INPUT = new IO::File;
	my @result;
	my $newData = 0;
	
	
	#====================================
	#	*** Copy/Open the Best Track file ***
	#===================================
	
	
	#--- Copy file to work directory
	
	copy ("$btkDir/$$rhData{file}", "$workDir/$$rhData{file}") || return -6;
	
	#--- Open file
	
	my $returnCode = generalUtils::openFile($workDir, $$rhData{file}, "read", \$INPUT);
	
	if ($returnCode != 1) {
	
		print ($LOG "Error(btkData::getData): Opening $workDir/$$rhData{file} - returnCode $returnCode.\n");
		return $returnCode;
		
	}
	
	
	#===============================================
	#	*** Extract Data from the Best Track file ***
	#================================================
	
	
	while (defined (my $line = <$INPUT>)) {
	
		#--- Initialize result
		
		@result = ();
	
		#--- remove end of line character
		
		chomp $line;
		
		#--- split the line between commas
		
		@result = split(/\,/, $line);
		
		if (@result <= 0) {
			
				next;	#this line is blank go to next line
				
		}
		
		#--- Remove all spaces
		
		foreach my $element (@result) {
		
			$element =~ s/\s+//g;
		
		}
		
		#--- Check that the results one or more numerical digits
		#--- Check that the dtg element's format is valid
		
		if (defined($result[2]) && ($result[2] =~ /[0-9]+/) && (generalUtils::validDateFormat($result[2]) == 1)) {
			
				my $dtg = $result[2];
				
				#======================================================
				#	*** Initalize data structure using the dtg
				#======================================================
	
		
				if (! (defined ($$rhData{hValues}->{$dtg}))) {
			
					foreach my $index (keys(%objIndex)) {
			
						$$rhData{hValues}->{$dtg}[$objIndex{$index}] = undef; 
				
					}
				}
				
				checkParam($result[8], \$$rhData{hValues}->{$dtg}[$objIndex{wind}]);
				checkParam($result[9], \$$rhData{hValues}->{$dtg}[$objIndex{intensity}]);
				checkParam($result[19], \$$rhData{hValues}->{$dtg}[$objIndex{rmw}]);
				checkParam($result[7], \$$rhData{hValues}->{$dtg}[$objIndex{longitude}]);
				checkParam($result[6], \$$rhData{hValues}->{$dtg}[$objIndex{latitude}]);
				
				
				#===================================================================
				# *** Return Success if either $intensity or $maxWind is defined
				#===================================================================
				
				if (defined ($$rhData{hValues}->{$dtg}[$objIndex{intensity}]) || defined ($$rhData{hValues}->{$dtg}[$objIndex{rmw}])) {
				
					$newData = 1;
				
				}
				
		}
		else {
		
			print ($LOG "Error(btkData::getData): Format problem with the date time group - $result[2].\n");
			next;	#Error - format problem with the data/time group
			
		}
	
	}
	
	close $INPUT;
	return $newData;
	
}


#--------------------------------------------------------------------------
# Subroutine: checkParam
#
# Purpose: Checks if a number parsed from the best track file is defined
#			and contains 1 or more digits.  If these checks are successful
#			the best track value is added to the data structure.
#
# Inputs: $btkValue - (number) - A value read from the best track file
#		  $rValue - (reference to a value) - stores $btkValue if valid
#
# Called by: getData
#
# Calls:  none	
#---------------------------------------------------------------------------

sub checkParam {

	my ($btkValue, $rValue) = @_;

	if (defined($btkValue) && $btkValue =~ /[0-9]+/) {
					
				$$rValue = $btkValue;
				return;
				
	}
	
	$$rValue = undef;

}




#======================================
# *** Reformat Data for plotting ***
#======================================




#--------------------------------------------------------
# Subroutine: mergeBtkData
#
# Purpose: Merge best track data from a best track data object into
#			a 2 dimensional array where the first row of the array
#			contains dtg's in the format YYYYMMDDHH
#
# Inputs:	$rhData - (ref data object) - reference to a best track data object
#			$raValues - (ref array) - reference to a 2 dimensional array
#			$type - (string) - type of btk data to place in array
#				 			   valid values are "wind", "rmw", "intensity", 
#								"latitude", "longitude"
#			$shift - (number) - hours the best track data should
#					 			be shifted.  
#
#					ex: btk data at 2003060712
#						shift 2 
#						btk data would be added to the values array at 2003060800
# 
#--------------------------------------------------------
sub mergeBtkData {
	
	my ($rhData, $raValues, $type, $shift, $LOG) = @_;
	
	
	#--- Redirect $LOG to STDOUT if it is not defined by the caller
	
	if (!defined $LOG) {
	
		$LOG = new IO::File;
		$LOG->open(">&STDOUT");
	
	}
	
	
	#==========================================
	# *** Check parameters
	#===========================================
	
	
	#--- Check for a valid $type definition
	
	if (! defined($type) || ! defined($objIndex{$type})) {
	
		print ($LOG  "Error(btkData::mergeBtkData): Undefined input parameter.\n");
		return -1; #Error incorrect type or type not found
	
	}
	
	unless (defined $rhData && defined $raValues) {
	
		print ($LOG  "Error(btkData::mergeBtkData): Undefined input parameter.\n");
		return -1;
	
	}
	
	
	#==========================================
	# *** Merge
	#===========================================
	
	
	my $num = @{$raValues};
	my $count = 0;
	
	
	foreach my $DTG (@{$$raValues[0]}) {
	
		#--- set the $shifted DTG value 
	
		my $shiftedDTG;

		if (!defined $shift || $shift == 0) {
		
			$shiftedDTG = $DTG;
		
		}
		else {
			
			$shiftedDTG = generalUtils::nextTime($DTG, $shift);
		
		}
		
		#--- Add the correct values to the array
		
		if (generalUtils::validDateFormat($shiftedDTG) == 1 && defined ($$rhData{hValues}->{$shiftedDTG})) {
			
			$$raValues[$num][$count] = $$rhData{hValues}->{$shiftedDTG}->[$objIndex{$type}];
			
		}
		else {
		
			$$raValues[$num][$count] = undef;
		
		}
		
		$count++;
	
	}
	
	return 1;
	
}


#--------------------------------------------------------
# Subroutine: reformat
#
# Purpose: Takes a hash and reformats the data values for plotting.
#
#		   To reformat the hash:
#				1. Place the hash keys into an array.  This array
#				   is then sorted by dtg in ascending order 
#				2. Go through the array and add in any missing values 
#			    3. The sorted array of keys are used to access 
#				   the hash in an ordered way.  
#
# Inputs: $rhData - (ref to a hash)
#
#				The input hash uses the AMSU file names as keys
#				to arrays 
#	
#				Example ( {2003061712}, ([undef,998,1001], [undef, 30, 20]),
#						  {2003061718}, ([990,undef,1001], [undef, 20, 30]),
#						   .... )
#
# 		  $raValues - (ref to an empty 2 dimensional array)
#
#				The two 2-Dimensional array will be filled with
#				dtgs and associated btk values in ascending dtg order.
#
#				Example ( [2003061712, 2003061718],
#						  [undef,990]
#						  [998, undef]
#						  [1001,1001]
#					    ) 
#
#		  $type - (string) -  type of btk data to place in array.
#				  			  valid values are "wind", "rmw", "intensity", 
#							  "latitude", "longitude"	
#
# Calls: by_dtg - to sort the data into ascending dtg
#		 generalUtils::continuousDTG - takes an array of sorted DTG's and 
#										fill in any missing gaps
#--------------------------------------------------------
sub reformat {


	my ($rhData, $raValues, $raTypes, $LOG) = @_;
	
	my @aSorted = ();			# array to store ordered DTG's
	my @aCompleteDTG = ();		# array to store a continuous array of DTG's	
	my $value = 0; 				# keeps track of whether the array has any valid values	
	
	
	#==========================================
	#	*** Check input parameters ***
	#==========================================
	
	
	#--- Redirect $LOG to STDOUT if it is not defined by the caller
	
	if (!defined $LOG) {
	
		$LOG = new IO::File;
		$LOG->open(">&STDOUT");
	
	}
	
	if (defined($raTypes)) {
	
		foreach my $type (@{$raTypes}) {
		
			unless (defined($objIndex{$type})) {
			
				print ($LOG "Error(btkData::reformat): Incorrect type - $type entered.\n");
				return -1;
			
			}
			
		}
	
	}
	else {
	
		print ($LOG "Error(btkData::reformat): Undefined type array entered.\n");
		return -1;
	
	}
	
	unless (defined $rhData && defined $raValues) {
	
		print ($LOG "Error(btkData::reformat): Undefined input parameters.\n");
		return -1;
	
	}
	
	
	#=============================
	#	*** Sort DTG's ***
	#=============================
	
	
	#----------------------------------------------
	# Sort the keys in the hash (ie DTG) in ascending
	# order.  Place the sorted list of DTG's in an 
	# array so that chart results are sorted.
	#-----------------------------------------------
	
	
	@aSorted = sort by_dtg keys(%{$$rhData{hValues}});
	
	if (@aSorted == 0) {
	
		print ($LOG "Error(btkData::reformat): No DTG values in btkData data object.\n");
		return -1;
	
	}
	
	#---Make sure the Complele DTG array starts with some date at 0z
	
	my $firstDTG = $aSorted[0];
	
	if (substr($aSorted[0], 8,2) ne "00") {
	
		$firstDTG = substr($aSorted[0], 0, 8) . "00";
		unshift(@aSorted, $firstDTG);
		
	}
	
	generalUtils::continuousDTG(\@aSorted, \@aCompleteDTG, 6);
	
	
	#==================================
	#	*** Reformat Hash into a 2-D array ***
	#==================================
	
	
	#--------------------------------------------------------------------------
	# Reformat the hash into a 2-D array that can be plotted by GD graph.
	# Since new values were added to the sorted array so that every hour
	# would be represented by a tick mark on the graph, many of the entries
	# in the complete sorted array are not defined in the associated hash function.
	# So two cases can occur
	#			1. the DTG in the complete array is represented in the 
	#			   hash and the value is copied directly to the 2-D array
	#			2. the DTG in the complete array is NOT represented in the
	#			   hash so just enter an undefined value for each of the satellites.
	#-----------------------------------------------------------------------------
	
	foreach my $typeIndex ( 1 .. @{$raTypes}) {
	
		my $count = 0;
	
		foreach my $dtg (@aCompleteDTG) {
	
		
			#==============================================
			#	*** Initialize 2 dimensional array ***
			#==============================================
	
	
			$$raValues[0][$count] = $dtg;
		
			#--- If this is a new DTG then mark the data as undef
		
			if (!defined ($$rhData{hValues}->{$dtg})) {
		
				$$raValues[$typeIndex][$count] = undef;
				next;
			
			}
		
			#--- If this DTG has already been seen then 
			#--- copy the data that has been read into the array
			
			my $type = $$raTypes[$typeIndex - 1];
					
			if (defined ($$rhData{hValues}->{$dtg}[$objIndex{$type}])) {
						
				$$raValues[$typeIndex][$count] = $$rhData{hValues}->{$dtg}[$objIndex{$type}];
				$value = 1;
					
			}
			else {
						
				$$raValues[$typeIndex][$count] = undef;
						
			}
		
			$count++;			
	
		}
	
	}
	
	if ($value != 1) {
	
		$raValues= undef;
		print ($LOG "Error(btkData::reformat): No values added to passed in 2 dimensional array.\n");
		return -1;			#no valid data to plot	
	
	}
	
	return 1; 			#success there is some defined data to plot
						#that is none zero

}

#--------------------------------------------------------
# Subroutine:reformatDTG
#
# Purpose: Reformats an array of date/times for pretty plotting.
#			
#			Rule: If the time group is 0Z then plot entire
#				  date/time string (ex 2003092600)
#
#				  if the time group does not equal 0Z then only
#				  plot the time group (ex 06)
#
# Input: $raValues - (reference to an array) - array of dtg values
#
# Outputs: A reference to an array of modified dtg values based on
#			the rule listed above.
# 
#--------------------------------------------------------
sub reformatDTG {

	my ($raValues) = @_;
	my $dtg;
	
	
	foreach my $index (0 .. (@{$$raValues[0]} - 1)) {
		
		$dtg = $$raValues[0][$index];
		
		if (substr($dtg, 8, 2) eq "00") {
	
			$$raValues[0][$index] = substr($dtg, 4, 6);
			next;
		
		}
		
		$$raValues[0][$index] = substr($dtg, 8, 2);
		
	}
	

}

#--------------------------------------------------------
# Subroutine: by_dtg
#
# Purpose: Sorting routine used by the createPlots subroutine
#		   to arrange the AMSU text files in ascending order
#		   using the dtg within the files title.
#
# Called by: createPlots
#
# Outputs: -1 $a comes before $b
#		    0 does not matter
#			1 $b comes before $a
# 
#--------------------------------------------------------
sub by_dtg {
	$a <=> $b;
}




#======================================
# *** Accessor Methods ***
#======================================




#--------------------------------------------------------
# Subroutine: getIntensity
#
# Purpose: Returns a hash of dtg pointing to a hash of btk values
#
# Inputs: $_[0] - reference to a hash containing btk values
#		  $_[1] - reference to a hash containing btk values
#
# Outputs: none
#
#--------------------------------------------------------
sub getIntensity {

	#===========================
    # *** Define Variables
    #===========================
	
	
	my ($rhData, $hValues, $aSorted) = @_;
	my $dtg;
	
	#--- Define an array to access the hash in an ordered way
	
	@$aSorted = sort by_dtg keys(%{$$rhData{hValues}});
    
	foreach $dtg (keys(%{$$rhData{hValues}})) {
		
		$$hValues{$dtg}->{intensity} = $$rhData{hValues}->{$dtg}->[$objIndex{wind}];
		$$hValues{$dtg}->{latitude} = $$rhData{hValues}->{$dtg}->[$objIndex{latitude}];
		$$hValues{$dtg}->{longitude} = $$rhData{hValues}->{$dtg}->[$objIndex{longitude}];
			
	}


}


#--------------------------------------------------------
# Subroutine: printResults
#
# Purpose: Prints the Best Track data values stored in 
#			the best track object to the passed in file handle
#
# Inputs: $rhData - (ref data object) - Best Track data object
#		  $LOG - (ref to file handle) - handle to redirect output to
#
# Outputs: none
#
#--------------------------------------------------------
sub printResults {


	#===========================
    # *** Define Variables
    #===========================
	
	
	my ($rhData, $LOG) = @_;
    my $key;
	
	#--- Redirect $LOG to STDOUT if it is not defined by the caller
	
	if (!defined $LOG) {
	
		$LOG = new IO::File;
		$LOG->open(">&STDOUT");
	
	}
	
	if (!defined $rhData) {
	
		print ("Error(btkData::printResults): Undefined input parameter.\n");
		return -1;
	
	}
	

	#================================
    # *** Print to file handle $LOG
	#================================

	
	#--- Print to file handle $LOG
	
	printf ($LOG "DTG   Maxwind Intensity  RMW\n");
	
	foreach $key (keys(%{$$rhData{hValues}})) {
		
		print ($LOG "$key ");
			
		if (defined($$rhData{hValues}->{$key}->[0])) {
		
			print ($LOG "$$rhData{hValues}->{$key}->[0] ");
		
		}
		else {
		
			print ($LOG "-1 ");
		
		}
			
		if (defined($$rhData{hValues}->{$key}->[1])) {
		
			print ($LOG " $$rhData{hValues}->{$key}->[1]");
		
		}
		else {
		
			print ($LOG " -1");
		
		}
		
		if (defined($$rhData{hValues}->{$key}->[2])) {
		
			print ($LOG " $$rhData{hValues}->{$key}->[2]\n");
		
		}
		else {
		
			print ($LOG " -1\n");
		
		}
			
	}

}

1;
